@@ -1,14 +1,9 @@
 //! Serialization traits, serializers, and adapters.
 
-#[cfg(feature = "std")]
-pub mod adapters;
 pub mod serializers;
 
-use crate::{
-    Archive, ArchivePointee, ArchiveUnsized, Archived, Fallible, RelPtr, Serialize,
-    SerializeUnsized,
-};
-use core::{mem, slice};
+use crate::{Archive, ArchiveUnsized, Fallible, RelPtr, Serialize, SerializeUnsized};
+use core::{alloc::Layout, mem, ptr::NonNull, slice};
 
 /// A byte sink that knows where it is.
 ///
@@ -16,7 +11,8 @@ use core::{mem, slice};
 /// [`WriteSerializer`](serializers::WriteSerializer) to equip it with `Serializer`.
 ///
 /// It's important that the memory for archived objects is properly aligned before attempting to
-/// read objects out of it; use the [`Aligned`](crate::Aligned) wrapper if it's appropriate.
+/// read objects out of it; use an [`AlignedVec`](crate::AlignedVec) or the
+/// [`AlignedBytes`](crate::AlignedBytes) wrappers if they are appropriate.
 pub trait Serializer: Fallible {
     /// Returns the current position of the serializer.
     fn pos(&self) -> usize;
@@ -38,7 +34,7 @@ pub trait Serializer: Fallible {
     #[inline]
     fn align(&mut self, align: usize) -> Result<usize, Self::Error> {
         let mask = align - 1;
-        debug_assert!(align & mask == 0);
+        debug_assert_eq!(align & mask, 0);
 
         self.pad((align - (self.pos() & mask)) & mask)?;
         Ok(self.pos())
@@ -56,19 +52,19 @@ pub trait Serializer: Fallible {
     ///
     /// # Safety
     ///
-    /// The caller must guarantee that the serializer is aligned for the archived version of the
-    /// given type.
+    /// - `resolver` must be the result of serializing `value`
+    /// - The serializer must be aligned for a `T::Archived`
     unsafe fn resolve_aligned<T: Archive + ?Sized>(
         &mut self,
         value: &T,
         resolver: T::Resolver,
     ) -> Result<usize, Self::Error> {
         let pos = self.pos();
-        debug_assert!(pos & (mem::align_of::<T::Archived>() - 1) == 0);
+        debug_assert_eq!(pos & (mem::align_of::<T::Archived>() - 1), 0);
 
         let mut resolved = mem::MaybeUninit::<T::Archived>::uninit();
         resolved.as_mut_ptr().write_bytes(0, 1);
-        value.resolve(pos, resolver, &mut resolved);
+        value.resolve(pos, resolver, resolved.as_mut_ptr());
 
         let data = resolved.as_ptr().cast::<u8>();
         let len = mem::size_of::<T::Archived>();
@@ -86,12 +82,13 @@ pub trait Serializer: Fallible {
 
     /// Resolves the given reference with its resolver and writes the archived reference.
     ///
-    /// Returns the position of the written archived reference.
+    /// Returns the position of the written archived `RelPtr`.
     ///
     /// # Safety
     ///
-    /// The caller must guarantee that the serializer is aligned for the archived reference of the
-    /// given type.
+    /// - `metadata_resolver` must be the result of serializing the metadata of `value`
+    /// - `to` must be the position of the serialized `value` within the archive
+    /// - The serializer must be aligned for a `RelPtr<T::Archived>`
     unsafe fn resolve_unsized_aligned<T: ArchiveUnsized + ?Sized>(
         &mut self,
         value: &T,
@@ -99,11 +96,11 @@ pub trait Serializer: Fallible {
         metadata_resolver: T::MetadataResolver,
     ) -> Result<usize, Self::Error> {
         let from = self.pos();
-        debug_assert!(from & (mem::align_of::<RelPtr<T::Archived>>() - 1) == 0);
+        debug_assert_eq!(from & (mem::align_of::<RelPtr<T::Archived>>() - 1), 0);
 
         let mut resolved = mem::MaybeUninit::<RelPtr<T::Archived>>::uninit();
         resolved.as_mut_ptr().write_bytes(0, 1);
-        value.resolve_unsized(from, to, metadata_resolver, &mut resolved);
+        value.resolve_unsized(from, to, metadata_resolver, resolved.as_mut_ptr());
 
         let data = resolved.as_ptr().cast::<u8>();
         let len = mem::size_of::<RelPtr<T::Archived>>();
@@ -124,53 +121,68 @@ pub trait Serializer: Fallible {
     }
 }
 
-/// A serializer that can seek to an absolute position.
-pub trait SeekSerializer: Serializer {
-    /// Seeks the serializer to the given absolute position.
-    fn seek(&mut self, pos: usize) -> Result<(), Self::Error>;
+// Someday this can probably be replaced with alloc::Allocator
 
-    /// Archives the given value at the nearest available position. If the serializer is already
-    /// aligned, it will archive it at the current position.
-    fn serialize_front<T: Serialize<Self>>(&mut self, value: &T) -> Result<usize, Self::Error> {
-        self.align_for::<T::Archived>()?;
-        let pos = self.pos();
-        self.seek(pos + mem::size_of::<T::Archived>())?;
-        let resolver = value.serialize(self)?;
-        self.seek(pos)?;
-        unsafe {
-            self.resolve_aligned(value, resolver)?;
-        }
-        Ok(pos)
+/// A serializer that can allocate scratch space.
+pub trait ScratchSpace: Fallible {
+    /// Allocates scratch space of the requested size.
+    ///
+    /// # Safety
+    ///
+    /// `layout` must have non-zero size.
+    unsafe fn push_scratch(&mut self, layout: Layout) -> Result<NonNull<[u8]>, Self::Error>;
+
+    /// Deallocates previously allocated scratch space.
+    ///
+    /// # Safety
+    ///
+    /// - `ptr` must be the scratch memory last allocated with `push_scratch`.
+    /// - `layout` must be the same layout that was used to allocate that block of memory.
+    unsafe fn pop_scratch(&mut self, ptr: NonNull<u8>, layout: Layout) -> Result<(), Self::Error>;
+}
+
+/// A registry that tracks serialized shared memory.
+///
+/// This trait is required to serialize shared pointers.
+pub trait SharedSerializeRegistry: Fallible {
+    /// Gets the position of a previously-added shared pointer.
+    ///
+    /// Returns `None` if the pointer has not yet been added.
+    fn get_shared_ptr(&mut self, value: *const u8) -> Option<usize>;
+
+    /// Gets the position of a previously-added shared value.
+    ///
+    /// Returns `None` if the value has not yet been added.
+    #[inline]
+    fn get_shared<T: ?Sized>(&mut self, value: &T) -> Option<usize> {
+        self.get_shared_ptr(value as *const T as *const u8)
     }
 
-    /// Archives a reference to the given value at the nearest available position. If the serializer
-    /// is already aligned, it will archive it at the current position.
-    fn serialize_unsized_front<T: SerializeUnsized<Self> + ?Sized>(
+    /// Adds the position of a shared pointer to the registry.
+    fn add_shared_ptr(&mut self, value: *const u8, pos: usize) -> Result<(), Self::Error>;
+
+    /// Adds the position of a shared value to the registry.
+    #[inline]
+    fn add_shared<T: ?Sized>(&mut self, value: &T, pos: usize) -> Result<(), Self::Error> {
+        self.add_shared_ptr(value as *const T as *const u8, pos)
+    }
+
+    /// Archives the given shared value and returns its position. If the value has already been
+    /// added then it returns the position of the previously added value.
+    #[inline]
+    fn serialize_shared<T: SerializeUnsized<Self> + ?Sized>(
         &mut self,
         value: &T,
     ) -> Result<usize, Self::Error>
     where
-        T::Metadata: Serialize<Self>,
-        T::Archived: ArchivePointee<ArchivedMetadata = Archived<T::Metadata>>,
+        Self: Serializer,
     {
-        self.align_for::<RelPtr<T::Archived>>()?;
-        let pos = self.pos();
-        self.seek(pos + mem::size_of::<RelPtr<T::Archived>>())?;
-        let to = value.serialize_unsized(self)?;
-        let metadata_resolver = value.serialize_metadata(self)?;
-        self.seek(pos)?;
-        unsafe { self.resolve_unsized_aligned(value, to, metadata_resolver) }
+        if let Some(pos) = self.get_shared(value) {
+            Ok(pos)
+        } else {
+            let pos = value.serialize_unsized(self)?;
+            self.add_shared(value, pos)?;
+            Ok(pos)
+        }
     }
 }
-
-/// A serializer that supports serializing shared memory.
-///
-/// This serializer is required by shared pointers to serialize.
-pub trait SharedSerializer: Serializer {
-    /// Archives the given shared value and returns its position. If the value has already been
-    /// serialized then it returns the position of the previously serialized value.
-    fn serialize_shared<T: SerializeUnsized<Self> + ?Sized>(
-        &mut self,
-        value: &T,
-    ) -> Result<usize, Self::Error>;
-}
